/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * pro-common.h
 * Copyright (C) 2005 Simon Newton
 */


#include <sys/types.h> 
#include <stdint.h>

#define USER_CONFIG_SIZE	508
#define DMX_SIZE		513
#define FLASH_PAGE		64 

#define RCMODE_ALWAYS	0x00
#define RCMODE_CHANGE	0x01

#define PRO_SOM	0x7e
#define PRO_EOM	0xe7

#define PRO_ID_REPROG	0x01
#define PRO_ID_FLSREQ	0x02
#define PRO_ID_FLSREP	0x02
#define PRO_ID_PRMREQ 	0x03
#define PRO_ID_PRMREP 	0x03
#define PRO_ID_PRMSET	0x04
#define PRO_ID_RDMX	0x05
#define PRO_ID_SDMX	0x06
#define PRO_ID_RDM	0x07
#define PRO_ID_RCMODE	0x08
#define PRO_ID_COS	0x09
#define PRO_ID_SNOREQ	0x0A
#define PRO_ID_SNOREP	0x0A

typedef struct {

} pms_reprog ;


typedef struct {
	uint8_t page[FLASH_PAGE] ;
} pms_flsreq;


typedef struct {
	char res[4] ;
} pms_flsrep;


typedef struct {
	uint8_t dmx[DMX_SIZE] ;
} pms_dmx ;

typedef struct {
	uint8_t status;
	uint8_t dmx[DMX_SIZE] ;
} pms_rdmx ;


typedef struct {
	uint8_t dmx[DMX_SIZE] ;
} pms_rdm ;


typedef struct {
	uint8_t len ;
	uint8_t	len_hi ;
} pms_prmreq;

typedef struct {
	uint8_t firmv ;
	uint8_t firmv_hi;
	uint8_t brtm ;
	uint8_t mabtm;
	uint8_t rate;
	uint8_t user[USER_CONFIG_SIZE];
} pms_prmrep;


typedef struct {
	uint8_t	mode ;
} pms_rcmode;

typedef struct {
	uint8_t	start;
	uint8_t	changed[5];
	uint8_t data[40];
} pms_cos;


typedef struct {

} pms_snoreq;


typedef struct {
	uint8_t srno[4] ;
} pms_snorep;


typedef union {
		pms_dmx 		pmu_dmx ;
		pms_rdmx 		pmu_rdmx ;
		pms_rdm 		pmu_rdm ;
		pms_prmreq 		pmu_prmreq;
		pms_prmrep		pmu_prmrep;
		pms_prmrep		pmu_prmset;
		pms_rcmode		pmu_rcmode;
		pms_cos			pmu_cos;
		pms_snoreq		pmu_snoreq;
		pms_snorep		pmu_snorep;
		pms_reprog		pmu_reprog;
		pms_flsreq		pmu_flsreq;
		pms_flsrep		pmu_flsrep;
		
} pmu;


typedef struct {
	uint8_t som;
	uint8_t label;
	uint8_t len;
	uint8_t len_hi;
	pmu pm_pmu;
} promsg ;

#define pm_reprog	pm_pmu.pmu_reprog
#define pm_flsreq	pm_pmu.pmu_flsreq
#define pm_flsrep	pm_pmu.pmu_flsrep
#define pm_dmx		pm_pmu.pmu_dmx
#define pm_rdmx		pm_pmu.pmu_rdmx
#define pm_rdm		pm_pmu.pmu_rdm
#define pm_prmreq	pm_pmu.pmu_prmreq
#define pm_prmrep	pm_pmu.pmu_prmrep
#define pm_prmset	pm_pmu.pmu_prmset
#define pm_rcmode	pm_pmu.pmu_rcmode
#define pm_cos		pm_pmu.pmu_cos
#define pm_snoreq	pm_pmu.pmu_snoreq
#define pm_snorep	pm_pmu.pmu_snorep


typedef enum {
	PRO_PRMREP_H,
	PRO_FLSREP_H,
} pro_handler ;

typedef void* pro_device ;

typedef struct {
	int fd ;
	int verbose ;
	int (*prmrep_h)(pro_device dev, pmu *msg, int len) ;
	int (*flsrep_h)(pro_device dev, pmu *msg, int len) ;
	
} _pro_device ;


pro_device pro_connect(char *dev, int verbose) ;
int pro_disconnect(pro_device dev) ;
int pro_send_dmx(pro_device dev, uint8_t *buf, int len) ;
int pro_send_rdm(pro_device dev, uint8_t *buf, int len) ;
int pro_send_prmreq(pro_device dev, int usrsz) ;
int pro_send_snoreq(pro_device dev) ;
int pro_send_reprogram(pro_device dev) ;
int pro_send_flsreq(pro_device dev, uint8_t *buf, int len) ;


int pro_recv(pro_device dev, int timeout) ;
int pro_set_handler(pro_device dev, pro_handler hdl ,  int (*prmrep_h)(pro_device dev, pmu *msg, int len) ) ;


